import os
from operator import itemgetter
from typing import List, Tuple

import numpy as np
from natsort import natsorted

IMAGE_EXTENSIONS = {
    'jpg', 'jpeg', 'png', 'tiff', 'bmp'
}


def is_image(filename: str) -> bool:
    _, ext = os.path.splitext(filename)

    return ext.strip('.').lower() in IMAGE_EXTENSIONS


def get_grouped_images(path: str) -> List[Tuple[str, List[str]]]:
    result = []

    for root, _, files in os.walk(path):
        local_result = []

        for filename in files:
            if not is_image(filename):
                continue

            local_result.append(filename)

        local_result = natsorted(local_result)

        if local_result:
            result.append((os.path.relpath(root, path), local_result))

    return natsorted(result, key=itemgetter(0))


def consecutive(data: np.ndarray, step_size: int = 1) -> np.ndarray:
    return np.split(data, np.where(np.diff(data) != step_size)[0] + 1)


def rotate_left(image: np.ndarray) -> np.ndarray:
    return np.transpose(image, axes=[1, 0, 2])[::-1, :, :]


def rotate_right(image: np.ndarray) -> np.ndarray:
    return np.transpose(image, axes=[1, 0, 2])[:, ::-1, :]


def rotate(image: np.ndarray, how: str) -> np.ndarray:
    if how == 'left':
        return rotate_left(image)
    elif how == 'right':
        return rotate_right(image)
    else:
        raise ValueError(f'Unknown rotation specification, expected left or right')


def pad_vertical(image: np.ndarray, height: int) -> np.ndarray:
    delta_height = image.shape[0] - height

    if delta_height <= 0:
        return image

    top_pad = delta_height // 2
    bottom_pad = delta_height - top_pad

    return np.pad(
        image,
        pad_width=[
            (top_pad, bottom_pad),
            (0, 0),
            (0, 0)
        ],
        mode='constant'
    )



from typing import Iterable, Optional

import numpy as np
import skimage.color
import skimage.io

from lomosplit.utils import consecutive, rotate, pad_vertical


def process_image(
        file: str,
        luminosity_percentile: int,
        rotate_image: Optional[str],
        rotate_frame: Optional[str],
        frame_min_height: Optional[int],
        frame_max_height: Optional[int],
        adjust_to_max_height: bool
) -> Iterable[np.ndarray]:
    image = skimage.io.imread(file)

    if rotate_image == 'auto':
        if image.shape[0] > image.shape[1]:
            rotate_image = 'left'
        else:
            rotate_image = None

    if rotate_frame == 'auto':
        rotate_frame = 'right'

    if rotate_image is not None:
        image = rotate(image, how=rotate_image)

    luminosity_hist = skimage.color.rgb2gray(image).mean(axis=0)
    threshold = np.percentile(luminosity_hist, luminosity_percentile)

    mask = luminosity_hist > threshold
    mask_where = np.where(mask)[0]
    parts = consecutive(mask_where)
    height_to_adjust = max(parts, key=len)

    if frame_max_height is not None:
        height_to_adjust = frame_max_height

    for part in parts:
        frame = image[:, part, :]

        if rotate_frame is not None:
            frame = rotate(frame, how=rotate_frame)

        if frame_min_height is not None and frame.shape[0] < frame_min_height:
            continue

        if frame_max_height is not None and frame.shape[0] > frame_max_height:
            continue

        if adjust_to_max_height:
            frame = pad_vertical(frame, height_to_adjust)

        yield frame


def process_batch(
        files: Iterable[str],
        *args,
        **kwargs
) -> Iterable[np.ndarray]:
    for file in files:
        yield from process_image(file, *args, **kwargs)



import argparse
import os
import sys

import skimage.io

from lomosplit.image import process_batch
from lomosplit.utils import get_grouped_images


def main():
    parser = argparse.ArgumentParser(
        description='Utility for splitting LomoKino film scans'
    )

    parser.add_argument(
        'input',
        help='Input image or folder with images'
    )

    parser.add_argument(
        '-q', '--quiet',
        action='store_true',
        dest='quiet',
        help='Do not print anything'
    )

    parser.add_argument(
        '-o', '--output',
        action='store',
        dest='output',
        default=None,
        help='Output folder to store results'
    )

    parser.add_argument(
        '--template',
        action='store',
        dest='template',
        default='frame_{idx:d}',
        help='Frame filename template without extension'
    )

    parser.add_argument(
        '--format',
        action='store',
        dest='format',
        default='jpg',
        help='Frame format (jpg, jpeg or png)'
    )

    parser.add_argument(
        '--luminosity-percentile',
        action='store',
        dest='luminosity_percentile',
        type=int,
        default=10,
        help='Luminosity percentile used to split frames'
    )

    parser.add_argument(
        '--rotate-image',
        action='store',
        dest='rotate_image',
        default='auto',
        help='Rotate image before processing (left, right or auto)'
    )

    parser.add_argument(
        '--rotate-frame',
        action='store',
        dest='rotate_frame',
        default='auto',
        help='Rotate frame before processing (left, right or auto)'
    )

    parser.add_argument(
        '--frame-min-height',
        action='store',
        dest='frame_min_height',
        type=int,
        default=100,
        help='Minimal frame height'
    )

    parser.add_argument(
        '--frame-max-height',
        action='store',
        dest='frame_max_height',
        type=int,
        default=None,
        help='Maximal frame height'
    )

    parser.add_argument(
        '--adjust-to-max-height',
        action='store_true',
        dest='adjust_to_max_height',
        help='Adjust each frame to maximum frame height (per image)'
    )

    args = parser.parse_args()

    input_path = os.path.normpath(os.path.abspath(args.input))

    if not os.path.exists(args.input):
        print(f'{input_path} does not exist')
        sys.exit(1)

    if os.path.isdir(input_path):
        grouped_images = get_grouped_images(input_path)

        if not grouped_images:
            print(f'{input_path} contains no images')
            sys.exit(1)
    else:
        grouped_images = [('', [os.path.basename(input_path)])]
        input_path = os.path.dirname(input_path)

    if args.output is not None:
        if os.path.exists(args.output):
            print(f'{args.output} already exists')
            sys.exit(2)

        output_path = os.path.normpath(os.path.abspath(args.output))
    else:
        for idx in range(100):
            output_path = os.path.abspath(os.path.join(os.getcwd(), f'lomosplit-output-{idx}'))

            if not os.path.exists(output_path):
                break
        else:
            print('Can not find appropriate name for output folder, please specify it')
            sys.exit(3)

    if args.format not in ('jpg', 'jpeg', 'png'):
        print('Frame format must be jpg, jpeg or png')
        sys.exit(3)

    template = f'{args.template}.{args.format}'

    for path, files in grouped_images:
        os.makedirs(os.path.join(output_path, path), exist_ok=True)

        for idx, frame in enumerate(process_batch(
                map(lambda x: os.path.join(input_path, path, x), files),
                luminosity_percentile=args.luminosity_percentile,
                rotate_image=args.rotate_image,
                rotate_frame=args.rotate_frame,
                frame_min_height=args.frame_min_height,
                frame_max_height=args.frame_max_height,
                adjust_to_max_height=args.adjust_to_max_height
        )):
            output_file = os.path.normpath(os.path.join(
                output_path,
                path,
                template.format(idx=idx)
            ))

            if not args.quiet:
                print(output_file)

            skimage.io.imsave(output_file, frame)


if __name__ == '__main__':
    main()



from setuptools import setup

with open('README.md', 'r') as f:
    long_description = f.read()

setup(
    name='lomosplit',
    version='1.0.0',
    description='Utility for splitting LomoKino film scans',
    license='MIT',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Egor Malykh',
    author_email='fnk@fea.st',
    url='https://github.com/meownoid/lomosplit',
    packages=['lomosplit'],
    install_requires=[
        'natsort',
        'numpy',
        'scikit-image'
    ]
)
